//----------------------------------------------------------------------------
//  Project Colorhls
//
//  Copyright  1999. All Rights Reserved.
//
//  SUBSYSTEM:    Colorhls Application
//  FILE:         colorhlsdlgclient.cpp
//  AUTHOR:       Dieter Windau
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TColorhlsDlgClient (TDialog).
//
//----------------------------------------------------------------------------
#include <owlext/pch.h>
#pragma hdrstop

#include <owlext/colorhls.h>
#include <math.h>
#include <stdio.h>

#include "colorhlsapp.h"
#include "colorhlsdlgclient.h"

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TColorhlsDlgClient, TDialog)
//{{TColorhlsDlgClientRSP_TBL_BEGIN}}
  EV_WM_ERASEBKGND,
  EV_WM_HSCROLL,
  EV_WM_PAINT,
  EV_WM_MOUSEMOVE,
//{{TColorhlsDlgClientRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TColorhlsDlgClient Implementation}}

OWLEXT_USING_NAMESPACE

//--------------------------------------------------------
// TColorhlsDlgClient
// ~~~~~~~~~~
// Construction/Destruction handling.
//
TColorhlsDlgClient::TColorhlsDlgClient(TWindow* parent, TResId resId, TModule* module)
:
  TDialog(parent, resId, module)
{
  // INSERT>> Your constructor code here.
  HSlider = new THSlider(this, IDC_HUE);
  LSlider = new THSlider(this, IDC_LUMINANCE);
  SSlider = new THSlider(this, IDC_SATURATION);
  RGBStatic = new TStatic(this, IDC_RGB);
  HLSStatic = new TStatic(this, IDC_HLS);
  HiddenDC = 0;
  HiddenBitmap = 0;
}


TColorhlsDlgClient::~TColorhlsDlgClient()
{
  Destroy();

  // INSERT>> Your destructor code here.
}
#ifndef ICON_BIG
#define ICON_BIG 1
#endif
#ifndef ICON_SMALL
#define ICON_SMALL 0
#endif
void TColorhlsDlgClient::SetupWindow()
{
	TDialog::SetupWindow();

  HSlider->SetRange(0, 360);
  HSlider->SetPosition(0);
  HSlider->SetRuler(30);
  LSlider->SetRange(0, 200);
  LSlider->SetPosition(100);
  LSlider->SetRuler(20);
  SSlider->SetRange(0, 200);
  SSlider->SetPosition(200);
  SSlider->SetRuler(20);

  ::GetWindowRect(GetDlgItem(IDC_CIRCLE_RECT), &CircleRect);
  ScreenToClient(CircleRect.TopLeft());
  ScreenToClient(CircleRect.BottomRight());

  TClientDC clientDC(HWindow);
  HiddenDC = new TMemoryDC();
  HiddenBitmap = new TBitmap(clientDC, CircleRect.Width(), CircleRect.Height());
  HiddenDC->SelectObject(*HiddenBitmap);
}

void TColorhlsDlgClient::DrawColorCircle(TDC& dc)
{
  static const double pi = 3.141592654;
  TRect circle(0, 0, CircleRect.Width(), CircleRect.Height());
  dc.FillRect(circle, ::GetSysColorBrush(COLOR_BTNFACE));

  // get the middle point
  //
  int cx = CircleRect.right / 2;
  int cy = CircleRect.bottom / 2;

  TColorHLS color;
  int hue = HSlider->GetPosition();
  color.SetHue(float(hue));
  color.SetLuminance(LSlider->GetPosition() / 200.0f);
  color.SetSaturation(SSlider->GetPosition() / 200.0f);

  // start point of the circle, against the clock (360 degree)
  //
  TPoint p1(circle.left, cy);

  // draw the color circle with pies
  //
  dc.SelectStockObject(NULL_PEN);
  for (int i = 360; i--;)
  {
    hue = hue + 1;
    if (hue > 360)
      hue -= 360;
    color.SetHue(float(hue));

    // calculate new point of the circle
    //
    TPoint p2(int(cx - cos(pi * i / 180.0) * 90.0),
              int(cy - sin(pi * i / 180.0) * 90.0));

    // draw pie
    //
    TBrush brush(TColor(color.GetValue()));
    dc.SelectObject(brush);
    dc.Pie(circle, p1, p2);
    dc.RestoreBrush();
    p1 = p2;
  }
}

bool TColorhlsDlgClient::EvEraseBkgnd(HDC dc)
{
  // INSERT>> Your code here.
  ExcludeClipRect(dc, CircleRect.left, CircleRect.top,
                      CircleRect.right, CircleRect.bottom);
  return TDialog::EvEraseBkgnd(dc);
}


void TColorhlsDlgClient::EvHScroll(uint scrollCode, uint thumbPos, THandle hWndCtl)
{
  TDialog::EvVScroll(scrollCode, thumbPos, hWndCtl);

  // INSERT>> Your code here.
  DrawColorCircle(*HiddenDC);
  TClientDC clientDC(HWindow);
  clientDC.BitBlt(CircleRect, *HiddenDC, TPoint(0, 0), SRCCOPY);
}

void TColorhlsDlgClient::EvPaint()
{
  TDialog::EvPaint();

  // INSERT>> Your code here.
  DrawColorCircle(*HiddenDC);
//  DrawGradient(*HiddenDC, 0,0, TColorHLS(RGB(255,0,0)), TColorHLS(RGB(0,0,255)),
//    CircleRect.Width(), CircleRect.Height());
  TClientDC clientDC(HWindow);
  clientDC.BitBlt(CircleRect, *HiddenDC, TPoint(0, 0), SRCCOPY);
}

void TColorhlsDlgClient::EvMouseMove(uint modKeys, TPoint& point)
{
  TDialog::EvMouseMove(modKeys, point);

  // INSERT>> Your code here.
  TRegion rgn(CircleRect, TRegion::Ellipse);
  if (rgn.Contains(point))
  {
    // color circle hit
    TClientDC dc(HWindow);
    UpdateColorInformation(dc.GetPixel(point));
  }
  else
  {
    RGBStatic->SetText(_T(""));
    HLSStatic->SetText(_T(""));
  }
}

void TColorhlsDlgClient::UpdateColorInformation(TColor color)
{
  TColorHLS hlsColor(color);

  TCHAR stringRGB[50];
  sprintf(stringRGB, "%d\n%d\n%d",
    hlsColor.Red(), hlsColor.Green(), hlsColor.Blue());
  RGBStatic->SetText(stringRGB);

  TCHAR stringHLS[50];
  sprintf(stringHLS, "%.1f \xB0\n%.3f %%\n%.3f %%",
    hlsColor.GetHue(), hlsColor.GetLuminance(), hlsColor.GetSaturation());
  HLSStatic->SetText(stringHLS);
}
//
